
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Random;
import java.util.Scanner;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
/**
 *
 * @author Martin Zvara
 */
public class Planner {

    static File clingo, domain;
    static Translator translator;
    static String[][] names;

    public static String findIntModels(ArrayList<ArrayList<String>> plans) {
        int tmp;
        for (int i = 0; i < plans.size(); i++) {
            for (int j = i + 1; j < plans.size(); j++) {
                for (int action = 0; action < plans.get(i).size(); action++) {
                    tmp = plans.get(i).get(action).substring(9).indexOf(',');
                    if (tmp != -1) {
                        if (plans.get(i).get(action).substring(0, tmp).equals(plans.get(j).get(action).substring(0, tmp))
                                && !plans.get(i).get(action).substring(tmp).equals(plans.get(j).get(action).substring(tmp))) {
                            return i + "," + j + "," + action; 
                        }
                    } else {
                        return "-1,-1,-1";
                    }
                }
            }
        }
        return "-1,-1,-1";
    }

    public static int chooseAction(String action1, String action2) {
        int choice = 3;
        while (choice > 2 || choice < 1) {
            System.out.println("Choose action:");
            System.out.println("1. " + action1);
            System.out.println("2. " + action2);
            Scanner s = new Scanner(System.in);
            choice = s.nextInt();
        }
        return choice;
    }

    public static ArrayList<String> findPlan(File domain, File plan) {
        ArrayList<String> result = new ArrayList<>();
        ArrayList<TreeMap<Integer, ArrayList<String>>> fluents = new ArrayList<>();
        ArrayList<ArrayList<String>> occurs = new ArrayList<>();
        TreeMap<Integer, String> sentences = new TreeMap<>();
        Random rnd = new Random();
        int pCounter = -1;
        try {
            String[] cmd = new String[]{clingo.getAbsolutePath(), domain.getAbsolutePath(), plan.getAbsolutePath(), "-n 0"};
            Process p = Runtime.getRuntime().exec(cmd);

            String line = "";
            Scanner s = new Scanner(p.getInputStream());
            int state;
            while (s.hasNextLine()) {
                line = s.next();
                if (line.contains("SATISFIABLE") || line.contains("UNSATISFIABLE")) {
                    break;
                }
                if (line.contains("Answer")) {
                    occurs.add(new ArrayList<String>());
                    fluents.add(new TreeMap<Integer, ArrayList<String>>());
                    pCounter++;
                }
                if (line.contains("occurs")) {
                    occurs.get(pCounter).add(line);
                }
                if (line.contains("holds")) {
                    state = Integer.parseInt(line.charAt(6) + "");
                    if (fluents.get(pCounter).containsKey(state)) {
                        fluents.get(pCounter).get(state).add(line.replace(line.charAt(6), '0'));
                    } else {
                        fluents.get(pCounter).put(state, new ArrayList<String>());
                        fluents.get(pCounter).get(state).add(line.replace(line.charAt(6), '0'));
                    }
                }
            }
            if (pCounter == -1) {
                return result;
            } else {
                if (occurs.size() == 1) {
                    pCounter = rnd.nextInt(fluents.size()); 
                    for (String action : occurs.get(pCounter)) {
                        translator.makeSentence(action, sentences);
                    }
                } else {
                    String models = findIntModels(occurs);
                    if (models.equals("-1,-1,-1") || rnd.nextBoolean()) {
                        pCounter = rnd.nextInt(fluents.size());
                        for (String action : occurs.get(pCounter)) {
                            translator.makeSentence(action, sentences);
                        }
                    } else {
                        pCounter = Integer.valueOf(models.split(",")[0]);
                        for (int i = 0; i < occurs.get(pCounter).size(); i++) {
                            if (i == Integer.valueOf(models.split(",")[2])) {
                                TreeMap<Integer, String> choices = new TreeMap<>();
                                String action1 = occurs.get(Integer.valueOf(models.split(",")[0])).get(i).replaceAll(models.split(",")[2], "1");
                                translator.makeSentence(action1, choices);
                                String action2 = occurs.get(Integer.valueOf(models.split(",")[1])).get(i).replaceAll(models.split(",")[2], "2");
                                translator.makeSentence(action2, choices);
                                pCounter = chooseAction(choices.get(1), choices.get(2)) == 1 ? Integer.valueOf(models.split(",")[0]) : Integer.valueOf(models.split(",")[1]);
                                translator.makeSentence(occurs.get(pCounter).get(i), sentences);
                            } else {
                                translator.makeSentence(occurs.get(pCounter).get(i), sentences);
                            }
                        }
                    }
                }

                if (!fluents.get(pCounter).isEmpty()) {
                    result.addAll(fluents.get(pCounter).get(fluents.get(pCounter).lastKey()));
                    for (String sentence : sentences.values()) {
                        System.out.println(sentence);
                    }
                    return result;
                }
            }
        } catch (IOException ex) {
            Logger.getLogger(Planner.class.getName()).log(Level.SEVERE, null, ex);
        }
        return result;
    }

    /**
     * This method generate logic program according given non complete logic program.
     *
     * @param plan - non complete plan
     * @param initially - initial fluents
     * @param counter - length of plan
     * @return updated plan
     */
    public static File generatePlan(File plan, ArrayList<String> initially, int counter) {
        if (initially != null) {
            File ePlan = new File("e" + plan.getName());
            Scanner s;
            try {
                ePlan.createNewFile();
                String line;
                s = new Scanner(plan);
                PrintWriter writer = new PrintWriter(new BufferedWriter(new FileWriter(ePlan)));
                while (s.hasNextLine()) {
                    line = s.nextLine();
                    if (line.contains("INITIAL STATE")) {
                        writer.write(line + "\n");
                        writer.write(s.nextLine() + "\n");
                        for (String fluent : initially) {
                            writer.write(fluent + ".\n");
                        }
                    } else if (line.contains("#const")) {  
                        writer.write(line + counter + ".\n");
                    } else {
                        writer.write(line + "\n");
                    }
                }
                s.close();
                writer.close();
                return ePlan;

            } catch (FileNotFoundException ex) {
                Logger.getLogger(Planner.class.getName()).log(Level.SEVERE, null, ex);
            } catch (IOException ex) {
                Logger.getLogger(Planner.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            return plan;
        }
        return null;
    }

    static void init() {
        names = new String[][]{{"AbsentationA.dl"}, {"ViolationOfInterdictionn.dl"}, {"Transition-Proposal.dl"},
        {"FirstFunctionOfDonor.dl"}, {"Reaction.dl"}, {"Guidance.dl"}, {"Struggle.dl"}, {"Branding.dl"},
        {"Victory.dl"}, {"Return.dl"}, {"UnfoundedClaims.dl"}, {"Recognition.dl"}, {"WeddingA.dl", "WeddingB.dl"}};
        clingo = new File("clingo1.exe");
        domain = new File("domain.dl");
        translator = new Translator();
        translator.addVerbPostModifier("say", "to");
        translator.addVerbPostModifier("move", "to");
        translator.addVerbPostModifier("give", "to");
        translator.addVerbPostModifier("fight", "with");
        translator.addVerbPostModifier("stay", "in");
        translator.addVerbPreModifier("stay", "to");
        translator.addFluentRepresentation("actorAt", "stay");
        translator.addFluentRepresentation("have", "");
        translator.addFeature("princess", "Gender", 2);
        translator.addFeature("queen", "Gender", 2);
        translator.addFeature("blacksmith", "Gender", 1);
        translator.addFeature("king", "Gender", 1);
        translator.addFeature("monk", "Gender", 1);
        translator.addFeature("princ", "Gender", 1);
        translator.addFeature("ogre", "Gender", 3);

    }

    public static void main(String[] args) {
        init();
        ArrayList<ArrayList<String>> fluents = new ArrayList<>();
        fluents.add(new ArrayList<String>());
        Random r = new Random();

        for (int i = 0; i < names.length; i++) {
            int j = 0;
            fluents.add(new ArrayList<String>());
            while (fluents.get(i + 1).isEmpty()) {
                fluents.get(i + 1).addAll(findPlan(domain, generatePlan(new File(names[i][r.nextInt(names[i].length)]), fluents.get(i), j++)));
            }
        }
    }
}
