
import java.util.ArrayList;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import simplenlg.features.Feature;
import simplenlg.features.Tense;
import simplenlg.framework.NLGFactory;
import simplenlg.lexicon.Lexicon;
import simplenlg.phrasespec.NPPhraseSpec;
import simplenlg.phrasespec.SPhraseSpec;
import simplenlg.phrasespec.VPPhraseSpec;
import simplenlg.realiser.english.Realiser;

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
/**
 *
 * @author Martin Zvara
 */
public class Translator {

    Lexicon lexicon;
    NLGFactory nlgFactory;
    Realiser realiser;
    HashMap<String, String> verbPostModifier;
    HashMap<String, String> verbPreModifier;
    HashMap<String, String> fluentRepresentation;

    public Translator() {
        verbPostModifier = new HashMap<>();
        verbPreModifier = new HashMap<>();
        fluentRepresentation = new HashMap<>();
        lexicon = Lexicon.getDefaultLexicon();
        nlgFactory = new NLGFactory(lexicon);
        realiser = new Realiser(lexicon);
    }

    public void addFeature(String word, String featureName, int feature) {
        lexicon.getWord(word).setFeature(featureName, feature);
    }
    
    public void addFluentRepresentation(String fluent, String representation){
        fluentRepresentation.put(fluent, representation);
    }

    public void addVerbPostModifier(String verb, String modifier){
        verbPostModifier.put(verb, modifier);
    }
    
    public void addVerbPreModifier(String verb, String modifier){
        verbPreModifier.put(verb, modifier);
    }
    /**
     *
     * @param person first(1), second(2), third(3)
     * @param singular
     * @param gender male(1), female(2), neutral(3)
     * @param category subject pronoun(1), object pronoun(2), possessive
     * adjective(3), possessive pronoun(4), reflexive pronoun(5)
     * @return adequate pronoun
     * @throws Exception when person, gender or category is unknown
     */
    public String getPronoun(int person, boolean singular, int gender, int category) throws Exception {
        switch (person) {
            case 1:
                if (singular) {
                    switch (category) {
                        case 1:
                            return "i";
                        case 2:
                            return "me";
                        case 3:
                            return "my";
                        case 4:
                            return "mine";
                        case 5:
                            return "myself";
                        default:
                            throw new Exception("Unknown category");
                    }
                } else {
                    switch (category) {
                        case 1:
                            return "we";
                        case 2:
                            return "us";
                        case 3:
                            return "our";
                        case 4:
                            return "ours";
                        case 5:
                            return "ourselves";
                        default:
                            throw new Exception("Unknown category");
                    }
                }
            case 2:
                switch (category) {
                    case 1:
                        return "you";
                    case 2:
                        return "you";
                    case 3:
                        return "your";
                    case 4:
                        return "yours";
                    case 5:
                        if (singular) {
                            return "yourself";
                        } else {
                            return "yourselves";
                        }
                    default:
                        throw new Exception("Unknown category");
                }
            case 3:
                if (singular) {
                    switch (gender) {
                        case 1:
                            switch (category) {
                                case 1:
                                    return "he";
                                case 2:
                                    return "him";
                                case 3:
                                    return "his";
                                case 4:
                                    return "his";
                                case 5:
                                    return "himself";
                                default:
                                    throw new Exception("Unknown category");
                            }
                        case 2:
                            switch (category) {
                                case 1:
                                    return "she";
                                case 2:
                                    return "her";
                                case 3:
                                    return "her";
                                case 4:
                                    return "hers";
                                case 5:
                                    return "herself";
                                default:
                                    throw new Exception("Unknown category");
                            }
                        case 3:
                            switch (category) {
                                case 1:
                                    return "it";
                                case 2:
                                    return "it";
                                case 3:
                                    return "its";
                                case 4:
                                    return "";
                                case 5:
                                    return "itself";
                                default:
                                    throw new Exception("Unknown category");
                            }
                        default:
                            throw new Exception("Unknown gender");
                    }
                } else {
                    switch (category) {
                        case 1:
                            return "they";
                        case 2:
                            return "them";
                        case 3:
                            return "their";
                        case 4:
                            return "theirs";
                        case 5:
                            return "themselves";
                        default:
                            throw new Exception("Unknown category");
                    }
                }
            default:
                throw new Exception("Unknown person");
        }
    }

    private ArrayList<String> parse(String action) {
        ArrayList<String> sentence = new ArrayList<>();
        StringBuilder word = new StringBuilder();
        int i = 0, begin = 9, end = 9;
        while (end < action.length() - 1) {
            while (action.charAt(end) != ',' && action.charAt(end) != '(' && action.charAt(end) != ')') {
                end++;
            }
            if (i == 0) {
                word.append(action.subSequence(begin, end));
                i++;
            } else if (i == 1) {
                sentence.add(action.subSequence(begin, end).toString());
                sentence.add(word.toString());
                word.delete(0, word.length());
                i++;
            } else if (i == 2) {
                sentence.add(action.subSequence(begin, end).toString());
                i = 0;
            }
            end++;
            begin = end;
        }
        if (word.length() != 0) {
            sentence.add(word.toString());
            word.delete(0, word.length());
        }
        return sentence;
    }

    public void makeSentence(String action, TreeMap<Integer, String> sentences) {

        ArrayList<String> sentence = parse(action);
        SPhraseSpec p = nlgFactory.createClause();
        for (int j = 0; j < sentence.size(); j++) {
            if (j < 3) {
                if (j == 1) {
                    VPPhraseSpec verb = nlgFactory.createVerbPhrase(sentence.get(j));
                    if (verbPostModifier.containsKey(sentence.get(j))) {
                        verb.addPostModifier(verbPostModifier.get(sentence.get(j)));
                    }
                    p.setVerb(verb);
                } else if (j == 0) {
                    p.setSubject(sentence.get(j));
                } else if (j == 2) {
                    NPPhraseSpec obj = nlgFactory.createNounPhrase(sentence.get(j));
                    if (sentence.size() == 4) {
                        obj.addComplement(sentence.get(++j));
                    }
                    p.setObject(obj);
                }
            } else {
                try {
                    if (fluentRepresentation.containsKey(sentence.get(j + 1))) {

                        SPhraseSpec q = nlgFactory.createClause();
                        if(sentence.get(3).equals("blacksmith") || sentence.get(3).equals("prince")){
                            NPPhraseSpec subj = nlgFactory.createNounPhrase("him");
                            subj.setFeature(Feature.PRONOMINAL, true);
                            q.setSubject(subj);
                        }else{
                            NPPhraseSpec subj = nlgFactory.createNounPhrase(getPronoun(3, true, lexicon.getWord(sentence.get(3)).getFeatureAsInteger("Gender"), 2));
                            subj.setFeature(Feature.PRONOMINAL, true);
                            q.setSubject(subj);
                        }
                        String tmpVerb = fluentRepresentation.get(sentence.get(j + 1));
                        VPPhraseSpec verb = nlgFactory.createVerbPhrase(tmpVerb);
                        if (verbPostModifier.containsKey(tmpVerb)) {
                            verb.addPostModifier(verbPostModifier.get(tmpVerb));
                        }
                        if (verbPreModifier.containsKey(tmpVerb)) {
                            verb.addPreModifier(verbPreModifier.get(tmpVerb));
                        }
                        q.setVerb(verb);
                        q.setObject(sentence.get(j + 2));
                        if(sentence.get(0).equals("blacksmith") || sentence.get(0).equals("prince")){
                            q.setFeature(Feature.COMPLEMENTISER, "that he want from ");
                        }else{
                            q.setFeature(Feature.COMPLEMENTISER, "that " + getPronoun(3, true, lexicon.getWord(sentence.get(0)).getFeatureAsInteger("Gender"), 1) + " want from ");
                        }
                        p.addComplement(q);
                        break;
                    } else {
                        SPhraseSpec q = nlgFactory.createClause(sentence.get(j), sentence.get(j + 1), sentence.get(j + 2));
                        q.setFeature(Feature.COMPLEMENTISER, "that");
                        q.setFeature(Feature.TENSE, Tense.PAST);
                        p.addComplement(q);
                        break;
                    }
                } catch (Exception ex) {
                    Logger.getLogger(Translator.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
        sentences.put(action.charAt(7) - 48, realiser.realiseSentence(p));
    }
}
